@@ -174,7 +174,10 @@ class CCodeWriter:
     
     def use_label(self, lbl):
         self.labels_used[lbl] = 1
-
+        
+    def label_used(self, lbl):
+        return lbl in self.labels_used
+        
     def put_label(self, lbl):
         if lbl in self.labels_used:
             self.putln("%s:;" % lbl)
@@ -210,7 +213,7 @@ class CCodeWriter:
                 storage_class = "static"
         if storage_class:
             self.put("%s " % storage_class)
-        if visibility <> 'public':
+        if visibility != 'public':
             dll_linkage = None
         self.put(entry.type.declaration_code(entry.cname,
             dll_linkage = dll_linkage))
@@ -326,12 +329,17 @@ class CCodeWriter:
     def error_goto(self, pos):
         lbl = self.error_label
         self.use_label(lbl)
-        return "{%s = %s[%s]; %s = %s; goto %s;}" % (
+        if Options.c_line_in_traceback:
+            cinfo = " %s = %s;" % (Naming.clineno_cname, Naming.line_c_macro)
+        else:
+            cinfo = ""
+        return "{%s = %s[%s]; %s = %s;%s goto %s;}" % (
             Naming.filename_cname,
             Naming.filetable_cname,
             self.lookup_filename(pos[0]),
             Naming.lineno_cname,
             pos[1],
+            cinfo,
             lbl)
             
     def error_goto_if(self, cond, pos):
