@@ -2,7 +2,7 @@
 #   Cython Top Level
 #
 
-import os, sys
+import os, sys, re
 if sys.version_info[:2] < (2, 2):
     print >>sys.stderr, "Sorry, Cython requires Python 2.2 or later"
     sys.exit(1)
@@ -18,6 +18,7 @@ from Symtab import BuiltinScope, ModuleScope
 import Code
 from Cython.Utils import replace_suffix
 from Cython import Utils
+import Transform
 
 verbose = 0
 
@@ -48,13 +49,13 @@ class Context:
         # that module, provided its name is not a dotted name.
         debug_find_module = 0
         if debug_find_module:
-            print "Context.find_module: module_name =", module_name, \
-                "relative_to =", relative_to, "pos =", pos, "need_pxd =", need_pxd
+            print("Context.find_module: module_name = %s, relative_to = %s, pos = %s, need_pxd = %s" % (
+                    module_name, relative_to, pos, need_pxd))
         scope = None
         pxd_pathname = None
         if "." not in module_name and relative_to:
             if debug_find_module:
-                print "...trying relative import"
+                print("...trying relative import")
             scope = relative_to.lookup_submodule(module_name)
             if not scope:
                 qualified_name = relative_to.qualify_name(module_name)
@@ -63,28 +64,28 @@ class Context:
                     scope = relative_to.find_submodule(module_name)
         if not scope:
             if debug_find_module:
-                print "...trying absolute import"
+                print("...trying absolute import")
             scope = self
             for name in module_name.split("."):
                 scope = scope.find_submodule(name)
         if debug_find_module:
-            print "...scope =", scope
+            print("...scope =", scope)
         if not scope.pxd_file_loaded:
             if debug_find_module:
-                print "...pxd not loaded"
+                print("...pxd not loaded")
             scope.pxd_file_loaded = 1
             if not pxd_pathname:
                 if debug_find_module:
-                    print "...looking for pxd file"
+                    print("...looking for pxd file")
                 pxd_pathname = self.find_pxd_file(module_name, pos)
                 if debug_find_module:
-                    print "......found ", pxd_pathname
+                    print("......found ", pxd_pathname)
                 if not pxd_pathname and need_pxd:
                     error(pos, "'%s.pxd' not found" % module_name)
             if pxd_pathname:
                 try:
                     if debug_find_module:
-                        print "Context.find_module: Parsing", pxd_pathname
+                        print("Context.find_module: Parsing %s" % pxd_pathname)
                     pxd_tree = self.parse(pxd_pathname, scope.type_names, pxd = 1,
                                           full_module_name = module_name)
                     pxd_tree.analyse_declarations(scope)
@@ -165,7 +166,9 @@ class Context:
         cwd = os.getcwd()
 
         if full_module_name is None:
-            full_module_name, _ = os.path.splitext(source.replace('/', '.'))
+            full_module_name, _ = os.path.splitext(source)
+            full_module_name = re.sub(r'[\\/]', '.', full_module_name)
+            full_module_name = re.sub(r'[^\w.]', '_', full_module_name)
 
         source = os.path.join(cwd, source)
         
@@ -236,6 +239,7 @@ class CompilationOptions:
     include_path      [string]  Directories to search for include files
     output_file       string    Name of generated .c file
     generate_pxi      boolean   Generate .pxi file for public declarations
+    transforms        Transform.TransformSet Transforms to use on the parse tree
     
     Following options are experimental and only used on MacOSX:
     
@@ -306,6 +310,7 @@ def compile(source, options = None, c_compile = 0, c_link = 0,
 #------------------------------------------------------------------------
 
 def main(command_line = 0):
+
     args = sys.argv[1:]
     any_failures = 0
     if command_line:
@@ -342,7 +347,8 @@ default_options = dict(
     obj_only = 1,
     cplus = 0,
     output_file = None,
-    generate_pxi = 0)
+    generate_pxi = 0,
+    transforms = Transform.TransformSet())
     
 if sys.platform == "mac":
     from Cython.Mac.MacSystem import c_compile, c_link, CCompilerError
